import 'dart:io';
import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'package:pretty_dio_logger/pretty_dio_logger.dart';
import 'package:risecx_chat_client/models/configuration_model.dart';

class RespositoryBase {
  static RespositoryBase _instance;
  Dio client;

  RespositoryBase._() {
    print(
        'Starting a new Instance of RespositoryBase, connecting to ${ConfigurationModel.instance.urlApi}');
    client = new Dio(BaseOptions(
      baseUrl: ConfigurationModel.instance.urlApi,
      connectTimeout: 30000,
      receiveTimeout: 30000,
    ));

    client.interceptors.add(
      PrettyDioLogger(
        requestHeader: true,
        requestBody: true,
        responseBody: true,
        responseHeader: false,
        error: true,
        compact: true,
        maxWidth: 90,
      ),
    );
    _instance ??= this;
    createCertificate();
  }

  static RespositoryBase get instance {
    return _instance ?? RespositoryBase._();
  }

  createCertificate() {
    (client.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
        (client) {
      client.badCertificateCallback =
          (X509Certificate cert, String host, int port) {
        return true;
      };
    };
  }

  setHeader(Map<String, dynamic> header) {
    client.options.headers.addAll(header);
  }

  static void dispose() {
    _instance = null;
  }
}
