library risecx_chat_client;

import 'package:flushbar/flushbar.dart';
import 'package:flutter/material.dart';
import 'package:risecx_chat_client/models/configuration_model.dart';
import 'package:risecx_chat_client/models/user_model.dart';
import 'package:risecx_chat_client/widgets/chat/programs/programs.dart';
import 'package:risecx_chat_client/widgets/widgets.dart';
import 'blocs/blocs.dart';

class RisecxChatClient extends StatefulWidget {
  final dynamic user;
  final dynamic options;
  final Map<String, dynamic> initConversationTrigger;
  final Function onConnectionStatusChange;
  const RisecxChatClient(
      {Key key,
      @required this.user,
      @required this.options,
      this.initConversationTrigger,
      this.onConnectionStatusChange})
      : super(key: key);

  @override
  _RisecxChatClientState createState() => _RisecxChatClientState();
}

class _RisecxChatClientState extends State<RisecxChatClient> {
  ChatBloc _chatBloc;
  int _currentIndex = 0;
  List<Widget> _children = [];

  @override
  void initState() {
    super.initState();
    ConfigurationModel configurationModel = ConfigurationModel(
        urlApi: widget.options['urlApi'],
        botName: widget.options['botName'],
        wsUrl: widget.options['wsUrl'],
        avatarImage: widget.options['avatarImage'] ??
            'assets/images/default_bot_avatar.jpg');
    _chatBloc =
        new ChatBloc(configurationModel, UserModel.fromJson(widget.user));

    _children = [
      ChatClientWidget(
        chatBloc: _chatBloc,
        user: UserModel.fromJson(widget.user),
        config: configurationModel,
        initConversationTrigger: widget.initConversationTrigger,
      ),
      FavoriteWidgets(_chatBloc, UserModel.fromJson(widget.user),
          onPayloadPress: _onFavoritePayloadPress),
      Container(
          child: ProgramsWidget(
              bloc: _chatBloc, user: UserModel.fromJson(widget.user))),
    ];
    _chatBloc.wsConnectionStatusStream.listen((event) {
      if (widget.onConnectionStatusChange == null) {
        switch (event) {
          case WS_CONNECTION_STATUS.DISCONNECTED:
            showConnInfoFlushbar(context, {
              'title': 'Disconnected',
              'message': 'Chat connection was lost'
            });
            break;
          case WS_CONNECTION_STATUS.RECONNECTING:
            showConnInfoFlushbar(context, {
              'title': 'Reconnecting',
              'message': 'Trying to reconnect to chat service'
            });
            break;
          case WS_CONNECTION_STATUS.CONNECTED:
            showConnInfoFlushbar(
                context, {'title': 'Connected', 'message': 'You are online'});
            break;
          case WS_CONNECTION_STATUS.CONNECTING:
            showConnInfoFlushbar(context, {
              'title': 'Connecting',
              'message': 'Reestablishing connection'
            });
            break;
          default:
        }
      } else
        widget.onConnectionStatusChange(event);
      print('connection status $event');
    });
  }

  _onTabTap(int tabIndex) {
    setState(() {
      _currentIndex = tabIndex;
    });
  }

  _onFavoritePayloadPress(payload) {
    print('_onFavoritePayloadPress $payload');
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        Expanded(child: _children[_currentIndex]),
        BottomNavigationBar(
          currentIndex: _currentIndex,
          onTap: _onTabTap,
          items: [
            BottomNavigationBarItem(
              icon: new Icon(Icons.chat),
              title: new Text('Chat'),
            ),
            BottomNavigationBarItem(
              icon: new Icon(Icons.favorite),
              title: new Text('Favorites'),
            ),
            BottomNavigationBarItem(
                icon: Icon(Icons.filter_list), title: Text('Programs'))
          ],
        ),
      ],
    );
  }

  void showConnInfoFlushbar(BuildContext context, Map<String, dynamic> params) {
    Flushbar(
      title: params['title'] ?? 'Connection Status',
      message: params['message'],
      flushbarPosition: FlushbarPosition.TOP,
      //backgroundColor: themeData.primaryColor,
      icon: Icon(
        Icons.info_outline,
        size: 28,
        color: Colors.white,
      ),
      duration: Duration(seconds: 4),
    )..show(context);
  }
}
