import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:risecx_chat_client/models/messages/button_message_model.dart';
import 'package:risecx_chat_client/models/messages/commons/commons.dart';
import 'package:risecx_chat_client/styles.dart';

class ButtonMessageWidget extends StatefulWidget {
  final dynamic message;
  final Function onPayloadPress;

  const ButtonMessageWidget({Key key, this.message, this.onPayloadPress})
      : super(key: key);

  @override
  _ButtonMessageWidgetState createState() => _ButtonMessageWidgetState();
}

class _ButtonMessageWidgetState extends State<ButtonMessageWidget> {
  ButtonMessageModel _buttonMessageModel;

  @override
  Widget build(BuildContext context) {
    _buttonMessageModel = ButtonMessageModel.fromJson(this.widget.message);
    return Card(
        child: Column(
      children: <Widget>[
        Padding(
          child: Text(_buttonMessageModel.attachment.payload.text,
              style: Styles.messageTitleTextStyle),
          padding: Styles.massageTitlePadding,
        ),
        Column(
          children: buttons(),
        )
      ],
    ));
  }

  List<Widget> buttons() {
    List<Widget> btnsList = new List();
    _buttonMessageModel.attachment.payload.buttons
        .forEach((button) => btnsList.add(SizedBox(
            width: double.infinity,
            child: OutlineButton(
              child: Text(button.title),
              onPressed: () {
                switch (button.type) {
                  case 'modal':
                    _showModal();
                    break;
                  default:
                    print('Unknow button type: ${button.type}');
                }
              },
            ))));
    return btnsList;
  }

  void _showModal() {
    showModalBottomSheet<void>(
        context: context,
        builder: (BuildContext context) {
          return new Column(
            mainAxisSize: MainAxisSize.min,
            children: getAnswers(),
          );
        });
  }

  List<Widget> getAnswers() {
    List<ListTile> answersList = new List();
    _buttonMessageModel.attachment.payload.modalAnswers.forEach((answer) {
      answersList.add(new ListTile(
        title: new Text(answer.title),
        leading: new Icon(Icons.chevron_right),
        onTap: () {
          Buttons button = new Buttons(
              title: answer.title, payload: answer.payload, type: 'modal');
          if (this.widget.onPayloadPress != null)
            this.widget.onPayloadPress(button);
          Navigator.pop(context);
        },
      ));
    });
    return answersList;
  }
}
