import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:photo_view/photo_view.dart';
import 'package:risecx_chat_client/models/messages/messages.dart';
import 'package:risecx_chat_client/widgets/commons/loading_indicator_widget.dart';
import 'package:transparent_image/transparent_image.dart';

class ImageMessageWidget extends StatelessWidget {
  final dynamic message;

  const ImageMessageWidget({Key key, this.message}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    ImageMessageModel imageModel = ImageMessageModel.fromJson(message);
    return Stack(
      alignment: Alignment.center,
      children: <Widget>[
        LoadingIndicatorWidget(),
        Center(
            child: GestureDetector(
          child: FadeInImage.memoryNetwork(
            image: imageModel.attachment.payload.url,
            placeholder: kTransparentImage,
          ),
          onTap: () {
            Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => FullScreenWrapper(
                          imageProvider:
                              NetworkImage(imageModel.attachment.payload.url),
                        )));
          },
        )),
      ],
    );
  }
}

class FullScreenWrapper extends StatelessWidget {
  const FullScreenWrapper(
      {this.imageProvider,
      this.loadingChild,
      this.backgroundDecoration,
      this.minScale,
      this.maxScale,
      this.initialScale,
      this.basePosition = Alignment.center});

  final ImageProvider imageProvider;
  final Widget loadingChild;
  final Decoration backgroundDecoration;
  final dynamic minScale;
  final dynamic maxScale;
  final dynamic initialScale;
  final Alignment basePosition;

  @override
  Widget build(BuildContext context) {
    return Container(
        constraints: BoxConstraints.expand(
          height: MediaQuery.of(context).size.height,
        ),
        child: PhotoView(
          imageProvider: imageProvider,
          loadingChild: loadingChild,
          backgroundDecoration: backgroundDecoration,
          minScale: minScale,
          maxScale: maxScale,
          initialScale: initialScale,
          basePosition: basePosition,
        ));
  }
}
