import 'package:flutter/material.dart';
import 'package:risecx_chat_client/blocs/blocs.dart';
import 'package:risecx_chat_client/models/models.dart';
import 'package:risecx_chat_client/widgets/chat/programs/program-item_widget.dart';
import 'package:risecx_chat_client/widgets/commons/empty_warning_widget.dart';

class ProgramsWidget extends StatefulWidget {
  final ChatBloc bloc;
  final UserModel user;

  const ProgramsWidget({Key key, this.bloc, this.user}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _ProgramsWidgetState();
}

class _ProgramsWidgetState extends State<ProgramsWidget>
    with AutomaticKeepAliveClientMixin {
  @override
  void initState() {
    super.initState();
    widget.bloc.getPrograms(widget.user);
  }

  void _toggleSubscription(ProgramModel program) {
    widget.bloc.subscribeUnsubscribe(program, widget.user);
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder(
      stream: widget.bloc.programsListStream,
      builder: (BuildContext context, AsyncSnapshot snapshot) {
        Widget _widget = IgnorePointer();
        if (snapshot.hasData) {
          if (snapshot.data.length > 0) {
            _widget = Padding(
              padding: EdgeInsets.only(top: 8),
              child: ListView.builder(
                itemCount: snapshot.data.length,
                itemBuilder: (context, index) => ProgramItem(
                  program: new ProgramModel.fromJson(snapshot.data[index]),
                  user: widget.user,
                  onTap: _toggleSubscription,
                ),
              ),
            );
          } else {
            _widget = EmptyWarningWidget(
              Icons.playlist_add_check,
              message: 'No Available Programs Yet',
            );
          }
        }
        return _widget;
      },
    );
  }

  @override
  bool get wantKeepAlive => true;
}
